import 'package:eyflutter_core/eyflutter_core.dart';
import 'package:eyflutter_uikit/uikit/state/state_utils.dart';
import 'package:flutter/material.dart';

mixin InputNumberController {
  int getInputValue();
}

class _InputNumberModel extends CusChangeNotifier {
  String inputTag = "input_number_tag";
  String inputValue = "0";
  int min = 0, max = 100;

  void onReductionClick() {
    var value = inputValue.toInt;
    if (value == 0) {
      return;
    }
    if (value > min) {
      inputValue = (value - 1).toString();
      notifierLazy();
    }
  }

  void onAddClick() {
    var value = inputValue.toInt;
    if (value < max) {
      inputValue = (value + 1).toString();
      notifierLazy();
    }
  }
}

class InputNumber extends StatelessWidget with InputNumberController {
  /// 组件是否启用
  final bool enable;

  /// 组件宽
  final double width;

  /// 组件高
  final double height;

  /// 初始值
  final int initValue;

  /// 最小值
  final int minValue;

  /// 最大值(默认100)
  final int maxValue;

  /// 组件构建完成后回调
  final Function(InputNumberController controller)? buildCall;

  final _InputNumberModel numberModel = _InputNumberModel();

  InputNumber({
    Key? key,
    this.enable = true,
    this.width = 120,
    this.height = 36,
    this.initValue = 0,
    this.minValue = 0,
    this.maxValue = 100,
    this.buildCall,
  }) : super(key: key) {
    if (minValue <= maxValue) {
      numberModel.min = minValue;
      numberModel.max = maxValue;
    }
    if (initValue >= 0) {
      if (initValue > maxValue) {
        numberModel.inputValue = maxValue.toString();
      } else {
        numberModel.inputValue = initValue.toString();
      }
    }
    if (buildCall != null) {
      buildCall!(this);
    }
  }

  @override
  Widget build(BuildContext context) {
    return Container(
      width: inputWidth,
      height: height,
      decoration: BoxDecoration(
        color: Colors.white,
        border: Border.all(color: Color(0xffDCDFE5)),
        borderRadius: BorderRadius.all(Radius.circular(3)),
      ),
      child: StateUtils.instance.subscribeNotifierWidget<_InputNumberModel>(numberModel, (context, model) {
        numberModel.inputValue = model.inputValue;
        return Row(
          mainAxisSize: MainAxisSize.min,
          children: [
            leftButton(model),
            buildInput(model),
            rightButton(model),
          ],
        );
      }),
    );
  }

  double get inputWidth {
    var minWidth = 2 * buttonWidth + 30;
    if (minWidth > width) {
      return minWidth;
    }
    return width;
  }

  Widget buildInput(_InputNumberModel model) {
    return Expanded(
        child: Container(
      alignment: Alignment.center,
      padding: EdgeInsets.only(left: 4, top: 2, right: 4, bottom: 2),
      color: enable ? Colors.transparent : Color(0xffF9FAFC),
      child: Text(
        model.inputValue,
        style: TextStyle(color: enable ? Color(0xff606266) : Color(0xffC1C4CB), fontSize: 13),
      ),
    ));
  }

  double get buttonWidth {
    if (width / 5 > 34) {
      return 34;
    } else if (width / 5 < 24) {
      return 24;
    } else {
      return width / 5;
    }
  }

  Widget leftButton(_InputNumberModel model) {
    return Container(
      width: buttonWidth,
      height: height,
      alignment: Alignment.center,
      decoration: BoxDecoration(
        border: Border(right: BorderSide(color: Color(0xffDCDFE5))),
      ),
      child: TextButton(
        onPressed: enable ? () => model.onReductionClick() : null,
        child: Icon(
          Icons.remove,
          color: enable ? Color(0xff606266) : Color(0xffE7EAEE),
          size: 20,
        ),
        style: ButtonStyle(
          padding: MaterialStateProperty.all(EdgeInsets.zero),
          minimumSize: MaterialStateProperty.all(Size(buttonWidth, height)),
          backgroundColor: MaterialStateProperty.all(Color(0xffF5F7FA)),
          shape: MaterialStateProperty.all(
            RoundedRectangleBorder(
              borderRadius: BorderRadius.only(topLeft: Radius.circular(2), bottomLeft: Radius.circular(2)),
            ),
          ),
        ),
      ),
    );
  }

  Widget rightButton(_InputNumberModel model) {
    return Container(
      width: buttonWidth,
      height: height,
      alignment: Alignment.center,
      decoration: BoxDecoration(
        border: Border(left: BorderSide(color: Color(0xffDCDFE5))),
      ),
      child: TextButton(
        onPressed: enable ? () => model.onAddClick() : null,
        child: Icon(
          Icons.add,
          color: enable ? Color(0xff606266) : Color(0xffE7EAEE),
          size: 20,
        ),
        style: ButtonStyle(
          padding: MaterialStateProperty.all(EdgeInsets.zero),
          minimumSize: MaterialStateProperty.all(Size(buttonWidth, height)),
          backgroundColor: MaterialStateProperty.all(Color(0xffF5F7FA)),
          shape: MaterialStateProperty.all(
            RoundedRectangleBorder(
              borderRadius: BorderRadius.only(topRight: Radius.circular(3), bottomRight: Radius.circular(3)),
            ),
          ),
        ),
      ),
    );
  }

  @override
  int getInputValue() {
    return numberModel.inputValue.toInt;
  }
}
